#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 12
#endif
#ifndef PFXDIST
   #define PFXDIST 256
#endif
#ifndef PFADIST
   #define PFADIST 256
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
/* Need to handle BETA=0 case by assigning y to zero outside of loop */
void ATL_UGEMV(ATL_CINT M0, ATL_CINT N, const TYPE *A, ATL_CINT lda0,
               const TYPE *X, TYPE *Y)
/*
 * 24x12 unrolled mvn_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N12=(N/12)*12, M=M0+M0, M24=((M0/24)*24)<<1,
            lda=lda0+lda0, lda12=lda*12;
   ATL_INT j;

   #ifdef BETA0
      for (j=0; j < M; j++)
         Y[j] = ATL_rzero;
   #endif
   for (j=N12; j; j -= 12, A += lda12, X += 24)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda,
                  *A5=A4+lda, *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda,
                  *A10=A9+lda, *A11=A10+lda;
      register float rx0=X[0], ix0=X[1], rx1=X[2], ix1=X[3], rx2=X[4], ix2=X[5],
                     rx3=X[6], ix3=X[7], rx4=X[8], ix4=X[9], rx5=X[10],
                     ix5=X[11], rx6=X[12], ix6=X[13], rx7=X[14], ix7=X[15],
                     rx8=X[16], ix8=X[17], rx9=X[18], ix9=X[19], rx10=X[20],
                     ix10=X[21], rx11=X[22], ix11=X[23];
      ATL_INT i;
      prefY(X+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2], ia1_0=A0[i+3],
                     ra2_0=A0[i+4], ia2_0=A0[i+5], ra3_0=A0[i+6], ia3_0=A0[i+7],
                     ra4_0=A0[i+8], ia4_0=A0[i+9], ra5_0=A0[i+10],
                     ia5_0=A0[i+11], ra6_0=A0[i+12], ia6_0=A0[i+13],
                     ra7_0=A0[i+14], ia7_0=A0[i+15], ra8_0=A0[i+16],
                     ia8_0=A0[i+17], ra9_0=A0[i+18], ia9_0=A0[i+19],
                     ra10_0=A0[i+20], ia10_0=A0[i+21], ra11_0=A0[i+22],
                     ia11_0=A0[i+23], ra12_0=A0[i+24], ia12_0=A0[i+25],
                     ra13_0=A0[i+26], ia13_0=A0[i+27], ra14_0=A0[i+28],
                     ia14_0=A0[i+29], ra15_0=A0[i+30], ia15_0=A0[i+31],
                     ra16_0=A0[i+32], ia16_0=A0[i+33], ra17_0=A0[i+34],
                     ia17_0=A0[i+35], ra18_0=A0[i+36], ia18_0=A0[i+37],
                     ra19_0=A0[i+38], ia19_0=A0[i+39], ra20_0=A0[i+40],
                     ia20_0=A0[i+41], ra21_0=A0[i+42], ia21_0=A0[i+43],
                     ra22_0=A0[i+44], ia22_0=A0[i+45], ra23_0=A0[i+46],
                     ia23_0=A0[i+47], ra0_1=A1[i+0], ia0_1=A1[i+1],
                     ra1_1=A1[i+2], ia1_1=A1[i+3], ra2_1=A1[i+4], ia2_1=A1[i+5],
                     ra3_1=A1[i+6], ia3_1=A1[i+7], ra4_1=A1[i+8], ia4_1=A1[i+9],
                     ra5_1=A1[i+10], ia5_1=A1[i+11], ra6_1=A1[i+12],
                     ia6_1=A1[i+13], ra7_1=A1[i+14], ia7_1=A1[i+15],
                     ra8_1=A1[i+16], ia8_1=A1[i+17], ra9_1=A1[i+18],
                     ia9_1=A1[i+19], ra10_1=A1[i+20], ia10_1=A1[i+21],
                     ra11_1=A1[i+22], ia11_1=A1[i+23], ra12_1=A1[i+24],
                     ia12_1=A1[i+25], ra13_1=A1[i+26], ia13_1=A1[i+27],
                     ra14_1=A1[i+28], ia14_1=A1[i+29], ra15_1=A1[i+30],
                     ia15_1=A1[i+31], ra16_1=A1[i+32], ia16_1=A1[i+33],
                     ra17_1=A1[i+34], ia17_1=A1[i+35], ra18_1=A1[i+36],
                     ia18_1=A1[i+37], ra19_1=A1[i+38], ia19_1=A1[i+39],
                     ra20_1=A1[i+40], ia20_1=A1[i+41], ra21_1=A1[i+42],
                     ia21_1=A1[i+43], ra22_1=A1[i+44], ia22_1=A1[i+45],
                     ra23_1=A1[i+46], ia23_1=A1[i+47], ra0_2=A2[i+0],
                     ia0_2=A2[i+1], ra1_2=A2[i+2], ia1_2=A2[i+3], ra2_2=A2[i+4],
                     ia2_2=A2[i+5], ra3_2=A2[i+6], ia3_2=A2[i+7], ra4_2=A2[i+8],
                     ia4_2=A2[i+9], ra5_2=A2[i+10], ia5_2=A2[i+11],
                     ra6_2=A2[i+12], ia6_2=A2[i+13], ra7_2=A2[i+14],
                     ia7_2=A2[i+15], ra8_2=A2[i+16], ia8_2=A2[i+17],
                     ra9_2=A2[i+18], ia9_2=A2[i+19], ra10_2=A2[i+20],
                     ia10_2=A2[i+21], ra11_2=A2[i+22], ia11_2=A2[i+23],
                     ra12_2=A2[i+24], ia12_2=A2[i+25], ra13_2=A2[i+26],
                     ia13_2=A2[i+27], ra14_2=A2[i+28], ia14_2=A2[i+29],
                     ra15_2=A2[i+30], ia15_2=A2[i+31], ra16_2=A2[i+32],
                     ia16_2=A2[i+33], ra17_2=A2[i+34], ia17_2=A2[i+35],
                     ra18_2=A2[i+36], ia18_2=A2[i+37], ra19_2=A2[i+38],
                     ia19_2=A2[i+39], ra20_2=A2[i+40], ia20_2=A2[i+41],
                     ra21_2=A2[i+42], ia21_2=A2[i+43], ra22_2=A2[i+44],
                     ia22_2=A2[i+45], ra23_2=A2[i+46], ia23_2=A2[i+47],
                     ra0_3=A3[i+0], ia0_3=A3[i+1], ra1_3=A3[i+2], ia1_3=A3[i+3],
                     ra2_3=A3[i+4], ia2_3=A3[i+5], ra3_3=A3[i+6], ia3_3=A3[i+7],
                     ra4_3=A3[i+8], ia4_3=A3[i+9], ra5_3=A3[i+10],
                     ia5_3=A3[i+11], ra6_3=A3[i+12], ia6_3=A3[i+13],
                     ra7_3=A3[i+14], ia7_3=A3[i+15], ra8_3=A3[i+16],
                     ia8_3=A3[i+17], ra9_3=A3[i+18], ia9_3=A3[i+19],
                     ra10_3=A3[i+20], ia10_3=A3[i+21], ra11_3=A3[i+22],
                     ia11_3=A3[i+23], ra12_3=A3[i+24], ia12_3=A3[i+25],
                     ra13_3=A3[i+26], ia13_3=A3[i+27], ra14_3=A3[i+28],
                     ia14_3=A3[i+29], ra15_3=A3[i+30], ia15_3=A3[i+31],
                     ra16_3=A3[i+32], ia16_3=A3[i+33], ra17_3=A3[i+34],
                     ia17_3=A3[i+35], ra18_3=A3[i+36], ia18_3=A3[i+37],
                     ra19_3=A3[i+38], ia19_3=A3[i+39], ra20_3=A3[i+40],
                     ia20_3=A3[i+41], ra21_3=A3[i+42], ia21_3=A3[i+43],
                     ra22_3=A3[i+44], ia22_3=A3[i+45], ra23_3=A3[i+46],
                     ia23_3=A3[i+47], ra0_4=A4[i+0], ia0_4=A4[i+1],
                     ra1_4=A4[i+2], ia1_4=A4[i+3], ra2_4=A4[i+4], ia2_4=A4[i+5],
                     ra3_4=A4[i+6], ia3_4=A4[i+7], ra4_4=A4[i+8], ia4_4=A4[i+9],
                     ra5_4=A4[i+10], ia5_4=A4[i+11], ra6_4=A4[i+12],
                     ia6_4=A4[i+13], ra7_4=A4[i+14], ia7_4=A4[i+15],
                     ra8_4=A4[i+16], ia8_4=A4[i+17], ra9_4=A4[i+18],
                     ia9_4=A4[i+19], ra10_4=A4[i+20], ia10_4=A4[i+21],
                     ra11_4=A4[i+22], ia11_4=A4[i+23], ra12_4=A4[i+24],
                     ia12_4=A4[i+25], ra13_4=A4[i+26], ia13_4=A4[i+27],
                     ra14_4=A4[i+28], ia14_4=A4[i+29], ra15_4=A4[i+30],
                     ia15_4=A4[i+31], ra16_4=A4[i+32], ia16_4=A4[i+33],
                     ra17_4=A4[i+34], ia17_4=A4[i+35], ra18_4=A4[i+36],
                     ia18_4=A4[i+37], ra19_4=A4[i+38], ia19_4=A4[i+39],
                     ra20_4=A4[i+40], ia20_4=A4[i+41], ra21_4=A4[i+42],
                     ia21_4=A4[i+43], ra22_4=A4[i+44], ia22_4=A4[i+45],
                     ra23_4=A4[i+46], ia23_4=A4[i+47], ra0_5=A5[i+0],
                     ia0_5=A5[i+1], ra1_5=A5[i+2], ia1_5=A5[i+3], ra2_5=A5[i+4],
                     ia2_5=A5[i+5], ra3_5=A5[i+6], ia3_5=A5[i+7], ra4_5=A5[i+8],
                     ia4_5=A5[i+9], ra5_5=A5[i+10], ia5_5=A5[i+11],
                     ra6_5=A5[i+12], ia6_5=A5[i+13], ra7_5=A5[i+14],
                     ia7_5=A5[i+15], ra8_5=A5[i+16], ia8_5=A5[i+17],
                     ra9_5=A5[i+18], ia9_5=A5[i+19], ra10_5=A5[i+20],
                     ia10_5=A5[i+21], ra11_5=A5[i+22], ia11_5=A5[i+23],
                     ra12_5=A5[i+24], ia12_5=A5[i+25], ra13_5=A5[i+26],
                     ia13_5=A5[i+27], ra14_5=A5[i+28], ia14_5=A5[i+29],
                     ra15_5=A5[i+30], ia15_5=A5[i+31], ra16_5=A5[i+32],
                     ia16_5=A5[i+33], ra17_5=A5[i+34], ia17_5=A5[i+35],
                     ra18_5=A5[i+36], ia18_5=A5[i+37], ra19_5=A5[i+38],
                     ia19_5=A5[i+39], ra20_5=A5[i+40], ia20_5=A5[i+41],
                     ra21_5=A5[i+42], ia21_5=A5[i+43], ra22_5=A5[i+44],
                     ia22_5=A5[i+45], ra23_5=A5[i+46], ia23_5=A5[i+47],
                     ra0_6=A6[i+0], ia0_6=A6[i+1], ra1_6=A6[i+2], ia1_6=A6[i+3],
                     ra2_6=A6[i+4], ia2_6=A6[i+5], ra3_6=A6[i+6], ia3_6=A6[i+7],
                     ra4_6=A6[i+8], ia4_6=A6[i+9], ra5_6=A6[i+10],
                     ia5_6=A6[i+11], ra6_6=A6[i+12], ia6_6=A6[i+13],
                     ra7_6=A6[i+14], ia7_6=A6[i+15], ra8_6=A6[i+16],
                     ia8_6=A6[i+17], ra9_6=A6[i+18], ia9_6=A6[i+19],
                     ra10_6=A6[i+20], ia10_6=A6[i+21], ra11_6=A6[i+22],
                     ia11_6=A6[i+23], ra12_6=A6[i+24], ia12_6=A6[i+25],
                     ra13_6=A6[i+26], ia13_6=A6[i+27], ra14_6=A6[i+28],
                     ia14_6=A6[i+29], ra15_6=A6[i+30], ia15_6=A6[i+31],
                     ra16_6=A6[i+32], ia16_6=A6[i+33], ra17_6=A6[i+34],
                     ia17_6=A6[i+35], ra18_6=A6[i+36], ia18_6=A6[i+37],
                     ra19_6=A6[i+38], ia19_6=A6[i+39], ra20_6=A6[i+40],
                     ia20_6=A6[i+41], ra21_6=A6[i+42], ia21_6=A6[i+43],
                     ra22_6=A6[i+44], ia22_6=A6[i+45], ra23_6=A6[i+46],
                     ia23_6=A6[i+47], ra0_7=A7[i+0], ia0_7=A7[i+1],
                     ra1_7=A7[i+2], ia1_7=A7[i+3], ra2_7=A7[i+4], ia2_7=A7[i+5],
                     ra3_7=A7[i+6], ia3_7=A7[i+7], ra4_7=A7[i+8], ia4_7=A7[i+9],
                     ra5_7=A7[i+10], ia5_7=A7[i+11], ra6_7=A7[i+12],
                     ia6_7=A7[i+13], ra7_7=A7[i+14], ia7_7=A7[i+15],
                     ra8_7=A7[i+16], ia8_7=A7[i+17], ra9_7=A7[i+18],
                     ia9_7=A7[i+19], ra10_7=A7[i+20], ia10_7=A7[i+21],
                     ra11_7=A7[i+22], ia11_7=A7[i+23], ra12_7=A7[i+24],
                     ia12_7=A7[i+25], ra13_7=A7[i+26], ia13_7=A7[i+27],
                     ra14_7=A7[i+28], ia14_7=A7[i+29], ra15_7=A7[i+30],
                     ia15_7=A7[i+31], ra16_7=A7[i+32], ia16_7=A7[i+33],
                     ra17_7=A7[i+34], ia17_7=A7[i+35], ra18_7=A7[i+36],
                     ia18_7=A7[i+37], ra19_7=A7[i+38], ia19_7=A7[i+39],
                     ra20_7=A7[i+40], ia20_7=A7[i+41], ra21_7=A7[i+42],
                     ia21_7=A7[i+43], ra22_7=A7[i+44], ia22_7=A7[i+45],
                     ra23_7=A7[i+46], ia23_7=A7[i+47], ra0_8=A8[i+0],
                     ia0_8=A8[i+1], ra1_8=A8[i+2], ia1_8=A8[i+3], ra2_8=A8[i+4],
                     ia2_8=A8[i+5], ra3_8=A8[i+6], ia3_8=A8[i+7], ra4_8=A8[i+8],
                     ia4_8=A8[i+9], ra5_8=A8[i+10], ia5_8=A8[i+11],
                     ra6_8=A8[i+12], ia6_8=A8[i+13], ra7_8=A8[i+14],
                     ia7_8=A8[i+15], ra8_8=A8[i+16], ia8_8=A8[i+17],
                     ra9_8=A8[i+18], ia9_8=A8[i+19], ra10_8=A8[i+20],
                     ia10_8=A8[i+21], ra11_8=A8[i+22], ia11_8=A8[i+23],
                     ra12_8=A8[i+24], ia12_8=A8[i+25], ra13_8=A8[i+26],
                     ia13_8=A8[i+27], ra14_8=A8[i+28], ia14_8=A8[i+29],
                     ra15_8=A8[i+30], ia15_8=A8[i+31], ra16_8=A8[i+32],
                     ia16_8=A8[i+33], ra17_8=A8[i+34], ia17_8=A8[i+35],
                     ra18_8=A8[i+36], ia18_8=A8[i+37], ra19_8=A8[i+38],
                     ia19_8=A8[i+39], ra20_8=A8[i+40], ia20_8=A8[i+41],
                     ra21_8=A8[i+42], ia21_8=A8[i+43], ra22_8=A8[i+44],
                     ia22_8=A8[i+45], ra23_8=A8[i+46], ia23_8=A8[i+47],
                     ra0_9=A9[i+0], ia0_9=A9[i+1], ra1_9=A9[i+2], ia1_9=A9[i+3],
                     ra2_9=A9[i+4], ia2_9=A9[i+5], ra3_9=A9[i+6], ia3_9=A9[i+7],
                     ra4_9=A9[i+8], ia4_9=A9[i+9], ra5_9=A9[i+10],
                     ia5_9=A9[i+11], ra6_9=A9[i+12], ia6_9=A9[i+13],
                     ra7_9=A9[i+14], ia7_9=A9[i+15], ra8_9=A9[i+16],
                     ia8_9=A9[i+17], ra9_9=A9[i+18], ia9_9=A9[i+19],
                     ra10_9=A9[i+20], ia10_9=A9[i+21], ra11_9=A9[i+22],
                     ia11_9=A9[i+23], ra12_9=A9[i+24], ia12_9=A9[i+25],
                     ra13_9=A9[i+26], ia13_9=A9[i+27], ra14_9=A9[i+28],
                     ia14_9=A9[i+29], ra15_9=A9[i+30], ia15_9=A9[i+31],
                     ra16_9=A9[i+32], ia16_9=A9[i+33], ra17_9=A9[i+34],
                     ia17_9=A9[i+35], ra18_9=A9[i+36], ia18_9=A9[i+37],
                     ra19_9=A9[i+38], ia19_9=A9[i+39], ra20_9=A9[i+40],
                     ia20_9=A9[i+41], ra21_9=A9[i+42], ia21_9=A9[i+43],
                     ra22_9=A9[i+44], ia22_9=A9[i+45], ra23_9=A9[i+46],
                     ia23_9=A9[i+47], ra0_10=A10[i+0], ia0_10=A10[i+1],
                     ra1_10=A10[i+2], ia1_10=A10[i+3], ra2_10=A10[i+4],
                     ia2_10=A10[i+5], ra3_10=A10[i+6], ia3_10=A10[i+7],
                     ra4_10=A10[i+8], ia4_10=A10[i+9], ra5_10=A10[i+10],
                     ia5_10=A10[i+11], ra6_10=A10[i+12], ia6_10=A10[i+13],
                     ra7_10=A10[i+14], ia7_10=A10[i+15], ra8_10=A10[i+16],
                     ia8_10=A10[i+17], ra9_10=A10[i+18], ia9_10=A10[i+19],
                     ra10_10=A10[i+20], ia10_10=A10[i+21], ra11_10=A10[i+22],
                     ia11_10=A10[i+23], ra12_10=A10[i+24], ia12_10=A10[i+25],
                     ra13_10=A10[i+26], ia13_10=A10[i+27], ra14_10=A10[i+28],
                     ia14_10=A10[i+29], ra15_10=A10[i+30], ia15_10=A10[i+31],
                     ra16_10=A10[i+32], ia16_10=A10[i+33], ra17_10=A10[i+34],
                     ia17_10=A10[i+35], ra18_10=A10[i+36], ia18_10=A10[i+37],
                     ra19_10=A10[i+38], ia19_10=A10[i+39], ra20_10=A10[i+40],
                     ia20_10=A10[i+41], ra21_10=A10[i+42], ia21_10=A10[i+43],
                     ra22_10=A10[i+44], ia22_10=A10[i+45], ra23_10=A10[i+46],
                     ia23_10=A10[i+47], ra0_11=A11[i+0], ia0_11=A11[i+1],
                     ra1_11=A11[i+2], ia1_11=A11[i+3], ra2_11=A11[i+4],
                     ia2_11=A11[i+5], ra3_11=A11[i+6], ia3_11=A11[i+7],
                     ra4_11=A11[i+8], ia4_11=A11[i+9], ra5_11=A11[i+10],
                     ia5_11=A11[i+11], ra6_11=A11[i+12], ia6_11=A11[i+13],
                     ra7_11=A11[i+14], ia7_11=A11[i+15], ra8_11=A11[i+16],
                     ia8_11=A11[i+17], ra9_11=A11[i+18], ia9_11=A11[i+19],
                     ra10_11=A11[i+20], ia10_11=A11[i+21], ra11_11=A11[i+22],
                     ia11_11=A11[i+23], ra12_11=A11[i+24], ia12_11=A11[i+25],
                     ra13_11=A11[i+26], ia13_11=A11[i+27], ra14_11=A11[i+28],
                     ia14_11=A11[i+29], ra15_11=A11[i+30], ia15_11=A11[i+31],
                     ra16_11=A11[i+32], ia16_11=A11[i+33], ra17_11=A11[i+34],
                     ia17_11=A11[i+35], ra18_11=A11[i+36], ia18_11=A11[i+37],
                     ra19_11=A11[i+38], ia19_11=A11[i+39], ra20_11=A11[i+40],
                     ia20_11=A11[i+41], ra21_11=A11[i+42], ia21_11=A11[i+43],
                     ra22_11=A11[i+44], ia22_11=A11[i+45], ra23_11=A11[i+46],
                     ia23_11=A11[i+47];
         register float ry0=Y[i+0], iy0=Y[i+1], ry1=Y[i+2], iy1=Y[i+3],
                        ry2=Y[i+4], iy2=Y[i+5], ry3=Y[i+6], iy3=Y[i+7],
                        ry4=Y[i+8], iy4=Y[i+9], ry5=Y[i+10], iy5=Y[i+11],
                        ry6=Y[i+12], iy6=Y[i+13], ry7=Y[i+14], iy7=Y[i+15],
                        ry8=Y[i+16], iy8=Y[i+17], ry9=Y[i+18], iy9=Y[i+19],
                        ry10=Y[i+20], iy10=Y[i+21], ry11=Y[i+22], iy11=Y[i+23],
                        ry12=Y[i+24], iy12=Y[i+25], ry13=Y[i+26], iy13=Y[i+27],
                        ry14=Y[i+28], iy14=Y[i+29], ry15=Y[i+30], iy15=Y[i+31],
                        ry16=Y[i+32], iy16=Y[i+33], ry17=Y[i+34], iy17=Y[i+35],
                        ry18=Y[i+36], iy18=Y[i+37], ry19=Y[i+38], iy19=Y[i+39],
                        ry20=Y[i+40], iy20=Y[i+41], ry21=Y[i+42], iy21=Y[i+43],
                        ry22=Y[i+44], iy22=Y[i+45], ry23=Y[i+46], iy23=Y[i+47];

         prefY(Y);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra1_0 * rx0;
         iy1 += ra1_0 * ix0;
         ry2 += ra2_0 * rx0;
         iy2 += ra2_0 * ix0;
         ry3 += ra3_0 * rx0;
         iy3 += ra3_0 * ix0;
         ry4 += ra4_0 * rx0;
         iy4 += ra4_0 * ix0;
         ry5 += ra5_0 * rx0;
         iy5 += ra5_0 * ix0;
         ry6 += ra6_0 * rx0;
         iy6 += ra6_0 * ix0;
         ry7 += ra7_0 * rx0;
         iy7 += ra7_0 * ix0;
         ry8 += ra8_0 * rx0;
         iy8 += ra8_0 * ix0;
         ry9 += ra9_0 * rx0;
         iy9 += ra9_0 * ix0;
         ry10 += ra10_0 * rx0;
         iy10 += ra10_0 * ix0;
         ry11 += ra11_0 * rx0;
         iy11 += ra11_0 * ix0;
         ry12 += ra12_0 * rx0;
         iy12 += ra12_0 * ix0;
         ry13 += ra13_0 * rx0;
         iy13 += ra13_0 * ix0;
         ry14 += ra14_0 * rx0;
         iy14 += ra14_0 * ix0;
         ry15 += ra15_0 * rx0;
         iy15 += ra15_0 * ix0;
         ry16 += ra16_0 * rx0;
         iy16 += ra16_0 * ix0;
         ry17 += ra17_0 * rx0;
         iy17 += ra17_0 * ix0;
         ry18 += ra18_0 * rx0;
         iy18 += ra18_0 * ix0;
         ry19 += ra19_0 * rx0;
         iy19 += ra19_0 * ix0;
         ry20 += ra20_0 * rx0;
         iy20 += ra20_0 * ix0;
         ry21 += ra21_0 * rx0;
         iy21 += ra21_0 * ix0;
         ry22 += ra22_0 * rx0;
         iy22 += ra22_0 * ix0;
         ry23 += ra23_0 * rx0;
         iy23 += ra23_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry1 -= ia1_0 * ix0;
         iy1 += ia1_0 * rx0;
         ry2 -= ia2_0 * ix0;
         iy2 += ia2_0 * rx0;
         ry3 -= ia3_0 * ix0;
         iy3 += ia3_0 * rx0;
         ry4 -= ia4_0 * ix0;
         iy4 += ia4_0 * rx0;
         ry5 -= ia5_0 * ix0;
         iy5 += ia5_0 * rx0;
         ry6 -= ia6_0 * ix0;
         iy6 += ia6_0 * rx0;
         ry7 -= ia7_0 * ix0;
         iy7 += ia7_0 * rx0;
         ry8 -= ia8_0 * ix0;
         iy8 += ia8_0 * rx0;
         prefA(A0);
         ry9 -= ia9_0 * ix0;
         iy9 += ia9_0 * rx0;
         ry10 -= ia10_0 * ix0;
         iy10 += ia10_0 * rx0;
         ry11 -= ia11_0 * ix0;
         iy11 += ia11_0 * rx0;
         ry12 -= ia12_0 * ix0;
         iy12 += ia12_0 * rx0;
         ry13 -= ia13_0 * ix0;
         iy13 += ia13_0 * rx0;
         ry14 -= ia14_0 * ix0;
         iy14 += ia14_0 * rx0;
         ry15 -= ia15_0 * ix0;
         iy15 += ia15_0 * rx0;
         ry16 -= ia16_0 * ix0;
         iy16 += ia16_0 * rx0;
         prefA(A0);
         ry17 -= ia17_0 * ix0;
         iy17 += ia17_0 * rx0;
         ry18 -= ia18_0 * ix0;
         iy18 += ia18_0 * rx0;
         ry19 -= ia19_0 * ix0;
         iy19 += ia19_0 * rx0;
         ry20 -= ia20_0 * ix0;
         iy20 += ia20_0 * rx0;
         ry21 -= ia21_0 * ix0;
         iy21 += ia21_0 * rx0;
         ry22 -= ia22_0 * ix0;
         iy22 += ia22_0 * rx0;
         ry23 -= ia23_0 * ix0;
         iy23 += ia23_0 * rx0;

         ry0 += ra0_1 * rx1;
         iy0 += ra0_1 * ix1;
         ry1 += ra1_1 * rx1;
         iy1 += ra1_1 * ix1;
         ry2 += ra2_1 * rx1;
         iy2 += ra2_1 * ix1;
         ry3 += ra3_1 * rx1;
         iy3 += ra3_1 * ix1;
         ry4 += ra4_1 * rx1;
         iy4 += ra4_1 * ix1;
         ry5 += ra5_1 * rx1;
         iy5 += ra5_1 * ix1;
         ry6 += ra6_1 * rx1;
         iy6 += ra6_1 * ix1;
         ry7 += ra7_1 * rx1;
         iy7 += ra7_1 * ix1;
         ry8 += ra8_1 * rx1;
         iy8 += ra8_1 * ix1;
         ry9 += ra9_1 * rx1;
         iy9 += ra9_1 * ix1;
         ry10 += ra10_1 * rx1;
         iy10 += ra10_1 * ix1;
         ry11 += ra11_1 * rx1;
         iy11 += ra11_1 * ix1;
         ry12 += ra12_1 * rx1;
         iy12 += ra12_1 * ix1;
         ry13 += ra13_1 * rx1;
         iy13 += ra13_1 * ix1;
         ry14 += ra14_1 * rx1;
         iy14 += ra14_1 * ix1;
         ry15 += ra15_1 * rx1;
         iy15 += ra15_1 * ix1;
         ry16 += ra16_1 * rx1;
         iy16 += ra16_1 * ix1;
         ry17 += ra17_1 * rx1;
         iy17 += ra17_1 * ix1;
         ry18 += ra18_1 * rx1;
         iy18 += ra18_1 * ix1;
         ry19 += ra19_1 * rx1;
         iy19 += ra19_1 * ix1;
         ry20 += ra20_1 * rx1;
         iy20 += ra20_1 * ix1;
         ry21 += ra21_1 * rx1;
         iy21 += ra21_1 * ix1;
         ry22 += ra22_1 * rx1;
         iy22 += ra22_1 * ix1;
         ry23 += ra23_1 * rx1;
         iy23 += ra23_1 * ix1;
         ry0 -= ia0_1 * ix1;
         iy0 += ia0_1 * rx1;
         prefA(A1);
         ry1 -= ia1_1 * ix1;
         iy1 += ia1_1 * rx1;
         ry2 -= ia2_1 * ix1;
         iy2 += ia2_1 * rx1;
         ry3 -= ia3_1 * ix1;
         iy3 += ia3_1 * rx1;
         ry4 -= ia4_1 * ix1;
         iy4 += ia4_1 * rx1;
         ry5 -= ia5_1 * ix1;
         iy5 += ia5_1 * rx1;
         ry6 -= ia6_1 * ix1;
         iy6 += ia6_1 * rx1;
         ry7 -= ia7_1 * ix1;
         iy7 += ia7_1 * rx1;
         ry8 -= ia8_1 * ix1;
         iy8 += ia8_1 * rx1;
         prefA(A1);
         ry9 -= ia9_1 * ix1;
         iy9 += ia9_1 * rx1;
         ry10 -= ia10_1 * ix1;
         iy10 += ia10_1 * rx1;
         ry11 -= ia11_1 * ix1;
         iy11 += ia11_1 * rx1;
         ry12 -= ia12_1 * ix1;
         iy12 += ia12_1 * rx1;
         ry13 -= ia13_1 * ix1;
         iy13 += ia13_1 * rx1;
         ry14 -= ia14_1 * ix1;
         iy14 += ia14_1 * rx1;
         ry15 -= ia15_1 * ix1;
         iy15 += ia15_1 * rx1;
         ry16 -= ia16_1 * ix1;
         iy16 += ia16_1 * rx1;
         prefA(A1);
         ry17 -= ia17_1 * ix1;
         iy17 += ia17_1 * rx1;
         ry18 -= ia18_1 * ix1;
         iy18 += ia18_1 * rx1;
         ry19 -= ia19_1 * ix1;
         iy19 += ia19_1 * rx1;
         ry20 -= ia20_1 * ix1;
         iy20 += ia20_1 * rx1;
         ry21 -= ia21_1 * ix1;
         iy21 += ia21_1 * rx1;
         ry22 -= ia22_1 * ix1;
         iy22 += ia22_1 * rx1;
         ry23 -= ia23_1 * ix1;
         iy23 += ia23_1 * rx1;

         ry0 += ra0_2 * rx2;
         iy0 += ra0_2 * ix2;
         ry1 += ra1_2 * rx2;
         iy1 += ra1_2 * ix2;
         ry2 += ra2_2 * rx2;
         iy2 += ra2_2 * ix2;
         ry3 += ra3_2 * rx2;
         iy3 += ra3_2 * ix2;
         ry4 += ra4_2 * rx2;
         iy4 += ra4_2 * ix2;
         ry5 += ra5_2 * rx2;
         iy5 += ra5_2 * ix2;
         ry6 += ra6_2 * rx2;
         iy6 += ra6_2 * ix2;
         ry7 += ra7_2 * rx2;
         iy7 += ra7_2 * ix2;
         ry8 += ra8_2 * rx2;
         iy8 += ra8_2 * ix2;
         ry9 += ra9_2 * rx2;
         iy9 += ra9_2 * ix2;
         ry10 += ra10_2 * rx2;
         iy10 += ra10_2 * ix2;
         ry11 += ra11_2 * rx2;
         iy11 += ra11_2 * ix2;
         ry12 += ra12_2 * rx2;
         iy12 += ra12_2 * ix2;
         ry13 += ra13_2 * rx2;
         iy13 += ra13_2 * ix2;
         ry14 += ra14_2 * rx2;
         iy14 += ra14_2 * ix2;
         ry15 += ra15_2 * rx2;
         iy15 += ra15_2 * ix2;
         ry16 += ra16_2 * rx2;
         iy16 += ra16_2 * ix2;
         ry17 += ra17_2 * rx2;
         iy17 += ra17_2 * ix2;
         ry18 += ra18_2 * rx2;
         iy18 += ra18_2 * ix2;
         ry19 += ra19_2 * rx2;
         iy19 += ra19_2 * ix2;
         ry20 += ra20_2 * rx2;
         iy20 += ra20_2 * ix2;
         ry21 += ra21_2 * rx2;
         iy21 += ra21_2 * ix2;
         ry22 += ra22_2 * rx2;
         iy22 += ra22_2 * ix2;
         ry23 += ra23_2 * rx2;
         iy23 += ra23_2 * ix2;
         ry0 -= ia0_2 * ix2;
         iy0 += ia0_2 * rx2;
         prefA(A2);
         ry1 -= ia1_2 * ix2;
         iy1 += ia1_2 * rx2;
         ry2 -= ia2_2 * ix2;
         iy2 += ia2_2 * rx2;
         ry3 -= ia3_2 * ix2;
         iy3 += ia3_2 * rx2;
         ry4 -= ia4_2 * ix2;
         iy4 += ia4_2 * rx2;
         ry5 -= ia5_2 * ix2;
         iy5 += ia5_2 * rx2;
         ry6 -= ia6_2 * ix2;
         iy6 += ia6_2 * rx2;
         ry7 -= ia7_2 * ix2;
         iy7 += ia7_2 * rx2;
         ry8 -= ia8_2 * ix2;
         iy8 += ia8_2 * rx2;
         prefA(A2);
         ry9 -= ia9_2 * ix2;
         iy9 += ia9_2 * rx2;
         ry10 -= ia10_2 * ix2;
         iy10 += ia10_2 * rx2;
         ry11 -= ia11_2 * ix2;
         iy11 += ia11_2 * rx2;
         ry12 -= ia12_2 * ix2;
         iy12 += ia12_2 * rx2;
         ry13 -= ia13_2 * ix2;
         iy13 += ia13_2 * rx2;
         ry14 -= ia14_2 * ix2;
         iy14 += ia14_2 * rx2;
         ry15 -= ia15_2 * ix2;
         iy15 += ia15_2 * rx2;
         ry16 -= ia16_2 * ix2;
         iy16 += ia16_2 * rx2;
         prefA(A2);
         ry17 -= ia17_2 * ix2;
         iy17 += ia17_2 * rx2;
         ry18 -= ia18_2 * ix2;
         iy18 += ia18_2 * rx2;
         ry19 -= ia19_2 * ix2;
         iy19 += ia19_2 * rx2;
         ry20 -= ia20_2 * ix2;
         iy20 += ia20_2 * rx2;
         ry21 -= ia21_2 * ix2;
         iy21 += ia21_2 * rx2;
         ry22 -= ia22_2 * ix2;
         iy22 += ia22_2 * rx2;
         ry23 -= ia23_2 * ix2;
         iy23 += ia23_2 * rx2;

         ry0 += ra0_3 * rx3;
         iy0 += ra0_3 * ix3;
         ry1 += ra1_3 * rx3;
         iy1 += ra1_3 * ix3;
         ry2 += ra2_3 * rx3;
         iy2 += ra2_3 * ix3;
         ry3 += ra3_3 * rx3;
         iy3 += ra3_3 * ix3;
         ry4 += ra4_3 * rx3;
         iy4 += ra4_3 * ix3;
         ry5 += ra5_3 * rx3;
         iy5 += ra5_3 * ix3;
         ry6 += ra6_3 * rx3;
         iy6 += ra6_3 * ix3;
         ry7 += ra7_3 * rx3;
         iy7 += ra7_3 * ix3;
         ry8 += ra8_3 * rx3;
         iy8 += ra8_3 * ix3;
         ry9 += ra9_3 * rx3;
         iy9 += ra9_3 * ix3;
         ry10 += ra10_3 * rx3;
         iy10 += ra10_3 * ix3;
         ry11 += ra11_3 * rx3;
         iy11 += ra11_3 * ix3;
         ry12 += ra12_3 * rx3;
         iy12 += ra12_3 * ix3;
         ry13 += ra13_3 * rx3;
         iy13 += ra13_3 * ix3;
         ry14 += ra14_3 * rx3;
         iy14 += ra14_3 * ix3;
         ry15 += ra15_3 * rx3;
         iy15 += ra15_3 * ix3;
         ry16 += ra16_3 * rx3;
         iy16 += ra16_3 * ix3;
         ry17 += ra17_3 * rx3;
         iy17 += ra17_3 * ix3;
         ry18 += ra18_3 * rx3;
         iy18 += ra18_3 * ix3;
         ry19 += ra19_3 * rx3;
         iy19 += ra19_3 * ix3;
         ry20 += ra20_3 * rx3;
         iy20 += ra20_3 * ix3;
         ry21 += ra21_3 * rx3;
         iy21 += ra21_3 * ix3;
         ry22 += ra22_3 * rx3;
         iy22 += ra22_3 * ix3;
         ry23 += ra23_3 * rx3;
         iy23 += ra23_3 * ix3;
         ry0 -= ia0_3 * ix3;
         iy0 += ia0_3 * rx3;
         prefA(A3);
         ry1 -= ia1_3 * ix3;
         iy1 += ia1_3 * rx3;
         ry2 -= ia2_3 * ix3;
         iy2 += ia2_3 * rx3;
         ry3 -= ia3_3 * ix3;
         iy3 += ia3_3 * rx3;
         ry4 -= ia4_3 * ix3;
         iy4 += ia4_3 * rx3;
         ry5 -= ia5_3 * ix3;
         iy5 += ia5_3 * rx3;
         ry6 -= ia6_3 * ix3;
         iy6 += ia6_3 * rx3;
         ry7 -= ia7_3 * ix3;
         iy7 += ia7_3 * rx3;
         ry8 -= ia8_3 * ix3;
         iy8 += ia8_3 * rx3;
         prefA(A3);
         ry9 -= ia9_3 * ix3;
         iy9 += ia9_3 * rx3;
         ry10 -= ia10_3 * ix3;
         iy10 += ia10_3 * rx3;
         ry11 -= ia11_3 * ix3;
         iy11 += ia11_3 * rx3;
         ry12 -= ia12_3 * ix3;
         iy12 += ia12_3 * rx3;
         ry13 -= ia13_3 * ix3;
         iy13 += ia13_3 * rx3;
         ry14 -= ia14_3 * ix3;
         iy14 += ia14_3 * rx3;
         ry15 -= ia15_3 * ix3;
         iy15 += ia15_3 * rx3;
         ry16 -= ia16_3 * ix3;
         iy16 += ia16_3 * rx3;
         prefA(A3);
         ry17 -= ia17_3 * ix3;
         iy17 += ia17_3 * rx3;
         ry18 -= ia18_3 * ix3;
         iy18 += ia18_3 * rx3;
         ry19 -= ia19_3 * ix3;
         iy19 += ia19_3 * rx3;
         ry20 -= ia20_3 * ix3;
         iy20 += ia20_3 * rx3;
         ry21 -= ia21_3 * ix3;
         iy21 += ia21_3 * rx3;
         ry22 -= ia22_3 * ix3;
         iy22 += ia22_3 * rx3;
         ry23 -= ia23_3 * ix3;
         iy23 += ia23_3 * rx3;

         ry0 += ra0_4 * rx4;
         iy0 += ra0_4 * ix4;
         ry1 += ra1_4 * rx4;
         iy1 += ra1_4 * ix4;
         ry2 += ra2_4 * rx4;
         iy2 += ra2_4 * ix4;
         ry3 += ra3_4 * rx4;
         iy3 += ra3_4 * ix4;
         ry4 += ra4_4 * rx4;
         iy4 += ra4_4 * ix4;
         ry5 += ra5_4 * rx4;
         iy5 += ra5_4 * ix4;
         ry6 += ra6_4 * rx4;
         iy6 += ra6_4 * ix4;
         ry7 += ra7_4 * rx4;
         iy7 += ra7_4 * ix4;
         ry8 += ra8_4 * rx4;
         iy8 += ra8_4 * ix4;
         ry9 += ra9_4 * rx4;
         iy9 += ra9_4 * ix4;
         ry10 += ra10_4 * rx4;
         iy10 += ra10_4 * ix4;
         ry11 += ra11_4 * rx4;
         iy11 += ra11_4 * ix4;
         ry12 += ra12_4 * rx4;
         iy12 += ra12_4 * ix4;
         ry13 += ra13_4 * rx4;
         iy13 += ra13_4 * ix4;
         ry14 += ra14_4 * rx4;
         iy14 += ra14_4 * ix4;
         ry15 += ra15_4 * rx4;
         iy15 += ra15_4 * ix4;
         ry16 += ra16_4 * rx4;
         iy16 += ra16_4 * ix4;
         ry17 += ra17_4 * rx4;
         iy17 += ra17_4 * ix4;
         ry18 += ra18_4 * rx4;
         iy18 += ra18_4 * ix4;
         ry19 += ra19_4 * rx4;
         iy19 += ra19_4 * ix4;
         ry20 += ra20_4 * rx4;
         iy20 += ra20_4 * ix4;
         ry21 += ra21_4 * rx4;
         iy21 += ra21_4 * ix4;
         ry22 += ra22_4 * rx4;
         iy22 += ra22_4 * ix4;
         ry23 += ra23_4 * rx4;
         iy23 += ra23_4 * ix4;
         ry0 -= ia0_4 * ix4;
         iy0 += ia0_4 * rx4;
         prefA(A4);
         ry1 -= ia1_4 * ix4;
         iy1 += ia1_4 * rx4;
         ry2 -= ia2_4 * ix4;
         iy2 += ia2_4 * rx4;
         ry3 -= ia3_4 * ix4;
         iy3 += ia3_4 * rx4;
         ry4 -= ia4_4 * ix4;
         iy4 += ia4_4 * rx4;
         ry5 -= ia5_4 * ix4;
         iy5 += ia5_4 * rx4;
         ry6 -= ia6_4 * ix4;
         iy6 += ia6_4 * rx4;
         ry7 -= ia7_4 * ix4;
         iy7 += ia7_4 * rx4;
         ry8 -= ia8_4 * ix4;
         iy8 += ia8_4 * rx4;
         prefA(A4);
         ry9 -= ia9_4 * ix4;
         iy9 += ia9_4 * rx4;
         ry10 -= ia10_4 * ix4;
         iy10 += ia10_4 * rx4;
         ry11 -= ia11_4 * ix4;
         iy11 += ia11_4 * rx4;
         ry12 -= ia12_4 * ix4;
         iy12 += ia12_4 * rx4;
         ry13 -= ia13_4 * ix4;
         iy13 += ia13_4 * rx4;
         ry14 -= ia14_4 * ix4;
         iy14 += ia14_4 * rx4;
         ry15 -= ia15_4 * ix4;
         iy15 += ia15_4 * rx4;
         ry16 -= ia16_4 * ix4;
         iy16 += ia16_4 * rx4;
         prefA(A4);
         ry17 -= ia17_4 * ix4;
         iy17 += ia17_4 * rx4;
         ry18 -= ia18_4 * ix4;
         iy18 += ia18_4 * rx4;
         ry19 -= ia19_4 * ix4;
         iy19 += ia19_4 * rx4;
         ry20 -= ia20_4 * ix4;
         iy20 += ia20_4 * rx4;
         ry21 -= ia21_4 * ix4;
         iy21 += ia21_4 * rx4;
         ry22 -= ia22_4 * ix4;
         iy22 += ia22_4 * rx4;
         ry23 -= ia23_4 * ix4;
         iy23 += ia23_4 * rx4;

         ry0 += ra0_5 * rx5;
         iy0 += ra0_5 * ix5;
         ry1 += ra1_5 * rx5;
         iy1 += ra1_5 * ix5;
         ry2 += ra2_5 * rx5;
         iy2 += ra2_5 * ix5;
         ry3 += ra3_5 * rx5;
         iy3 += ra3_5 * ix5;
         ry4 += ra4_5 * rx5;
         iy4 += ra4_5 * ix5;
         ry5 += ra5_5 * rx5;
         iy5 += ra5_5 * ix5;
         ry6 += ra6_5 * rx5;
         iy6 += ra6_5 * ix5;
         ry7 += ra7_5 * rx5;
         iy7 += ra7_5 * ix5;
         ry8 += ra8_5 * rx5;
         iy8 += ra8_5 * ix5;
         ry9 += ra9_5 * rx5;
         iy9 += ra9_5 * ix5;
         ry10 += ra10_5 * rx5;
         iy10 += ra10_5 * ix5;
         ry11 += ra11_5 * rx5;
         iy11 += ra11_5 * ix5;
         ry12 += ra12_5 * rx5;
         iy12 += ra12_5 * ix5;
         ry13 += ra13_5 * rx5;
         iy13 += ra13_5 * ix5;
         ry14 += ra14_5 * rx5;
         iy14 += ra14_5 * ix5;
         ry15 += ra15_5 * rx5;
         iy15 += ra15_5 * ix5;
         ry16 += ra16_5 * rx5;
         iy16 += ra16_5 * ix5;
         ry17 += ra17_5 * rx5;
         iy17 += ra17_5 * ix5;
         ry18 += ra18_5 * rx5;
         iy18 += ra18_5 * ix5;
         ry19 += ra19_5 * rx5;
         iy19 += ra19_5 * ix5;
         ry20 += ra20_5 * rx5;
         iy20 += ra20_5 * ix5;
         ry21 += ra21_5 * rx5;
         iy21 += ra21_5 * ix5;
         ry22 += ra22_5 * rx5;
         iy22 += ra22_5 * ix5;
         ry23 += ra23_5 * rx5;
         iy23 += ra23_5 * ix5;
         ry0 -= ia0_5 * ix5;
         iy0 += ia0_5 * rx5;
         prefA(A5);
         ry1 -= ia1_5 * ix5;
         iy1 += ia1_5 * rx5;
         ry2 -= ia2_5 * ix5;
         iy2 += ia2_5 * rx5;
         ry3 -= ia3_5 * ix5;
         iy3 += ia3_5 * rx5;
         ry4 -= ia4_5 * ix5;
         iy4 += ia4_5 * rx5;
         ry5 -= ia5_5 * ix5;
         iy5 += ia5_5 * rx5;
         ry6 -= ia6_5 * ix5;
         iy6 += ia6_5 * rx5;
         ry7 -= ia7_5 * ix5;
         iy7 += ia7_5 * rx5;
         ry8 -= ia8_5 * ix5;
         iy8 += ia8_5 * rx5;
         prefA(A5);
         ry9 -= ia9_5 * ix5;
         iy9 += ia9_5 * rx5;
         ry10 -= ia10_5 * ix5;
         iy10 += ia10_5 * rx5;
         ry11 -= ia11_5 * ix5;
         iy11 += ia11_5 * rx5;
         ry12 -= ia12_5 * ix5;
         iy12 += ia12_5 * rx5;
         ry13 -= ia13_5 * ix5;
         iy13 += ia13_5 * rx5;
         ry14 -= ia14_5 * ix5;
         iy14 += ia14_5 * rx5;
         ry15 -= ia15_5 * ix5;
         iy15 += ia15_5 * rx5;
         ry16 -= ia16_5 * ix5;
         iy16 += ia16_5 * rx5;
         prefA(A5);
         ry17 -= ia17_5 * ix5;
         iy17 += ia17_5 * rx5;
         ry18 -= ia18_5 * ix5;
         iy18 += ia18_5 * rx5;
         ry19 -= ia19_5 * ix5;
         iy19 += ia19_5 * rx5;
         ry20 -= ia20_5 * ix5;
         iy20 += ia20_5 * rx5;
         ry21 -= ia21_5 * ix5;
         iy21 += ia21_5 * rx5;
         ry22 -= ia22_5 * ix5;
         iy22 += ia22_5 * rx5;
         ry23 -= ia23_5 * ix5;
         iy23 += ia23_5 * rx5;

         ry0 += ra0_6 * rx6;
         iy0 += ra0_6 * ix6;
         ry1 += ra1_6 * rx6;
         iy1 += ra1_6 * ix6;
         ry2 += ra2_6 * rx6;
         iy2 += ra2_6 * ix6;
         ry3 += ra3_6 * rx6;
         iy3 += ra3_6 * ix6;
         ry4 += ra4_6 * rx6;
         iy4 += ra4_6 * ix6;
         ry5 += ra5_6 * rx6;
         iy5 += ra5_6 * ix6;
         ry6 += ra6_6 * rx6;
         iy6 += ra6_6 * ix6;
         ry7 += ra7_6 * rx6;
         iy7 += ra7_6 * ix6;
         ry8 += ra8_6 * rx6;
         iy8 += ra8_6 * ix6;
         ry9 += ra9_6 * rx6;
         iy9 += ra9_6 * ix6;
         ry10 += ra10_6 * rx6;
         iy10 += ra10_6 * ix6;
         ry11 += ra11_6 * rx6;
         iy11 += ra11_6 * ix6;
         ry12 += ra12_6 * rx6;
         iy12 += ra12_6 * ix6;
         ry13 += ra13_6 * rx6;
         iy13 += ra13_6 * ix6;
         ry14 += ra14_6 * rx6;
         iy14 += ra14_6 * ix6;
         ry15 += ra15_6 * rx6;
         iy15 += ra15_6 * ix6;
         ry16 += ra16_6 * rx6;
         iy16 += ra16_6 * ix6;
         ry17 += ra17_6 * rx6;
         iy17 += ra17_6 * ix6;
         ry18 += ra18_6 * rx6;
         iy18 += ra18_6 * ix6;
         ry19 += ra19_6 * rx6;
         iy19 += ra19_6 * ix6;
         ry20 += ra20_6 * rx6;
         iy20 += ra20_6 * ix6;
         ry21 += ra21_6 * rx6;
         iy21 += ra21_6 * ix6;
         ry22 += ra22_6 * rx6;
         iy22 += ra22_6 * ix6;
         ry23 += ra23_6 * rx6;
         iy23 += ra23_6 * ix6;
         ry0 -= ia0_6 * ix6;
         iy0 += ia0_6 * rx6;
         prefA(A6);
         ry1 -= ia1_6 * ix6;
         iy1 += ia1_6 * rx6;
         ry2 -= ia2_6 * ix6;
         iy2 += ia2_6 * rx6;
         ry3 -= ia3_6 * ix6;
         iy3 += ia3_6 * rx6;
         ry4 -= ia4_6 * ix6;
         iy4 += ia4_6 * rx6;
         ry5 -= ia5_6 * ix6;
         iy5 += ia5_6 * rx6;
         ry6 -= ia6_6 * ix6;
         iy6 += ia6_6 * rx6;
         ry7 -= ia7_6 * ix6;
         iy7 += ia7_6 * rx6;
         ry8 -= ia8_6 * ix6;
         iy8 += ia8_6 * rx6;
         prefA(A6);
         ry9 -= ia9_6 * ix6;
         iy9 += ia9_6 * rx6;
         ry10 -= ia10_6 * ix6;
         iy10 += ia10_6 * rx6;
         ry11 -= ia11_6 * ix6;
         iy11 += ia11_6 * rx6;
         ry12 -= ia12_6 * ix6;
         iy12 += ia12_6 * rx6;
         ry13 -= ia13_6 * ix6;
         iy13 += ia13_6 * rx6;
         ry14 -= ia14_6 * ix6;
         iy14 += ia14_6 * rx6;
         ry15 -= ia15_6 * ix6;
         iy15 += ia15_6 * rx6;
         ry16 -= ia16_6 * ix6;
         iy16 += ia16_6 * rx6;
         prefA(A6);
         ry17 -= ia17_6 * ix6;
         iy17 += ia17_6 * rx6;
         ry18 -= ia18_6 * ix6;
         iy18 += ia18_6 * rx6;
         ry19 -= ia19_6 * ix6;
         iy19 += ia19_6 * rx6;
         ry20 -= ia20_6 * ix6;
         iy20 += ia20_6 * rx6;
         ry21 -= ia21_6 * ix6;
         iy21 += ia21_6 * rx6;
         ry22 -= ia22_6 * ix6;
         iy22 += ia22_6 * rx6;
         ry23 -= ia23_6 * ix6;
         iy23 += ia23_6 * rx6;

         ry0 += ra0_7 * rx7;
         iy0 += ra0_7 * ix7;
         ry1 += ra1_7 * rx7;
         iy1 += ra1_7 * ix7;
         ry2 += ra2_7 * rx7;
         iy2 += ra2_7 * ix7;
         ry3 += ra3_7 * rx7;
         iy3 += ra3_7 * ix7;
         ry4 += ra4_7 * rx7;
         iy4 += ra4_7 * ix7;
         ry5 += ra5_7 * rx7;
         iy5 += ra5_7 * ix7;
         ry6 += ra6_7 * rx7;
         iy6 += ra6_7 * ix7;
         ry7 += ra7_7 * rx7;
         iy7 += ra7_7 * ix7;
         ry8 += ra8_7 * rx7;
         iy8 += ra8_7 * ix7;
         ry9 += ra9_7 * rx7;
         iy9 += ra9_7 * ix7;
         ry10 += ra10_7 * rx7;
         iy10 += ra10_7 * ix7;
         ry11 += ra11_7 * rx7;
         iy11 += ra11_7 * ix7;
         ry12 += ra12_7 * rx7;
         iy12 += ra12_7 * ix7;
         ry13 += ra13_7 * rx7;
         iy13 += ra13_7 * ix7;
         ry14 += ra14_7 * rx7;
         iy14 += ra14_7 * ix7;
         ry15 += ra15_7 * rx7;
         iy15 += ra15_7 * ix7;
         ry16 += ra16_7 * rx7;
         iy16 += ra16_7 * ix7;
         ry17 += ra17_7 * rx7;
         iy17 += ra17_7 * ix7;
         ry18 += ra18_7 * rx7;
         iy18 += ra18_7 * ix7;
         ry19 += ra19_7 * rx7;
         iy19 += ra19_7 * ix7;
         ry20 += ra20_7 * rx7;
         iy20 += ra20_7 * ix7;
         ry21 += ra21_7 * rx7;
         iy21 += ra21_7 * ix7;
         ry22 += ra22_7 * rx7;
         iy22 += ra22_7 * ix7;
         ry23 += ra23_7 * rx7;
         iy23 += ra23_7 * ix7;
         ry0 -= ia0_7 * ix7;
         iy0 += ia0_7 * rx7;
         prefA(A7);
         ry1 -= ia1_7 * ix7;
         iy1 += ia1_7 * rx7;
         ry2 -= ia2_7 * ix7;
         iy2 += ia2_7 * rx7;
         ry3 -= ia3_7 * ix7;
         iy3 += ia3_7 * rx7;
         ry4 -= ia4_7 * ix7;
         iy4 += ia4_7 * rx7;
         ry5 -= ia5_7 * ix7;
         iy5 += ia5_7 * rx7;
         ry6 -= ia6_7 * ix7;
         iy6 += ia6_7 * rx7;
         ry7 -= ia7_7 * ix7;
         iy7 += ia7_7 * rx7;
         ry8 -= ia8_7 * ix7;
         iy8 += ia8_7 * rx7;
         prefA(A7);
         ry9 -= ia9_7 * ix7;
         iy9 += ia9_7 * rx7;
         ry10 -= ia10_7 * ix7;
         iy10 += ia10_7 * rx7;
         ry11 -= ia11_7 * ix7;
         iy11 += ia11_7 * rx7;
         ry12 -= ia12_7 * ix7;
         iy12 += ia12_7 * rx7;
         ry13 -= ia13_7 * ix7;
         iy13 += ia13_7 * rx7;
         ry14 -= ia14_7 * ix7;
         iy14 += ia14_7 * rx7;
         ry15 -= ia15_7 * ix7;
         iy15 += ia15_7 * rx7;
         ry16 -= ia16_7 * ix7;
         iy16 += ia16_7 * rx7;
         prefA(A7);
         ry17 -= ia17_7 * ix7;
         iy17 += ia17_7 * rx7;
         ry18 -= ia18_7 * ix7;
         iy18 += ia18_7 * rx7;
         ry19 -= ia19_7 * ix7;
         iy19 += ia19_7 * rx7;
         ry20 -= ia20_7 * ix7;
         iy20 += ia20_7 * rx7;
         ry21 -= ia21_7 * ix7;
         iy21 += ia21_7 * rx7;
         ry22 -= ia22_7 * ix7;
         iy22 += ia22_7 * rx7;
         ry23 -= ia23_7 * ix7;
         iy23 += ia23_7 * rx7;

         ry0 += ra0_8 * rx8;
         iy0 += ra0_8 * ix8;
         ry1 += ra1_8 * rx8;
         iy1 += ra1_8 * ix8;
         ry2 += ra2_8 * rx8;
         iy2 += ra2_8 * ix8;
         ry3 += ra3_8 * rx8;
         iy3 += ra3_8 * ix8;
         ry4 += ra4_8 * rx8;
         iy4 += ra4_8 * ix8;
         ry5 += ra5_8 * rx8;
         iy5 += ra5_8 * ix8;
         ry6 += ra6_8 * rx8;
         iy6 += ra6_8 * ix8;
         ry7 += ra7_8 * rx8;
         iy7 += ra7_8 * ix8;
         ry8 += ra8_8 * rx8;
         iy8 += ra8_8 * ix8;
         ry9 += ra9_8 * rx8;
         iy9 += ra9_8 * ix8;
         ry10 += ra10_8 * rx8;
         iy10 += ra10_8 * ix8;
         ry11 += ra11_8 * rx8;
         iy11 += ra11_8 * ix8;
         ry12 += ra12_8 * rx8;
         iy12 += ra12_8 * ix8;
         ry13 += ra13_8 * rx8;
         iy13 += ra13_8 * ix8;
         ry14 += ra14_8 * rx8;
         iy14 += ra14_8 * ix8;
         ry15 += ra15_8 * rx8;
         iy15 += ra15_8 * ix8;
         ry16 += ra16_8 * rx8;
         iy16 += ra16_8 * ix8;
         ry17 += ra17_8 * rx8;
         iy17 += ra17_8 * ix8;
         ry18 += ra18_8 * rx8;
         iy18 += ra18_8 * ix8;
         ry19 += ra19_8 * rx8;
         iy19 += ra19_8 * ix8;
         ry20 += ra20_8 * rx8;
         iy20 += ra20_8 * ix8;
         ry21 += ra21_8 * rx8;
         iy21 += ra21_8 * ix8;
         ry22 += ra22_8 * rx8;
         iy22 += ra22_8 * ix8;
         ry23 += ra23_8 * rx8;
         iy23 += ra23_8 * ix8;
         ry0 -= ia0_8 * ix8;
         iy0 += ia0_8 * rx8;
         prefA(A8);
         ry1 -= ia1_8 * ix8;
         iy1 += ia1_8 * rx8;
         ry2 -= ia2_8 * ix8;
         iy2 += ia2_8 * rx8;
         ry3 -= ia3_8 * ix8;
         iy3 += ia3_8 * rx8;
         ry4 -= ia4_8 * ix8;
         iy4 += ia4_8 * rx8;
         ry5 -= ia5_8 * ix8;
         iy5 += ia5_8 * rx8;
         ry6 -= ia6_8 * ix8;
         iy6 += ia6_8 * rx8;
         ry7 -= ia7_8 * ix8;
         iy7 += ia7_8 * rx8;
         ry8 -= ia8_8 * ix8;
         iy8 += ia8_8 * rx8;
         prefA(A8);
         ry9 -= ia9_8 * ix8;
         iy9 += ia9_8 * rx8;
         ry10 -= ia10_8 * ix8;
         iy10 += ia10_8 * rx8;
         ry11 -= ia11_8 * ix8;
         iy11 += ia11_8 * rx8;
         ry12 -= ia12_8 * ix8;
         iy12 += ia12_8 * rx8;
         ry13 -= ia13_8 * ix8;
         iy13 += ia13_8 * rx8;
         ry14 -= ia14_8 * ix8;
         iy14 += ia14_8 * rx8;
         ry15 -= ia15_8 * ix8;
         iy15 += ia15_8 * rx8;
         ry16 -= ia16_8 * ix8;
         iy16 += ia16_8 * rx8;
         prefA(A8);
         ry17 -= ia17_8 * ix8;
         iy17 += ia17_8 * rx8;
         ry18 -= ia18_8 * ix8;
         iy18 += ia18_8 * rx8;
         ry19 -= ia19_8 * ix8;
         iy19 += ia19_8 * rx8;
         ry20 -= ia20_8 * ix8;
         iy20 += ia20_8 * rx8;
         ry21 -= ia21_8 * ix8;
         iy21 += ia21_8 * rx8;
         ry22 -= ia22_8 * ix8;
         iy22 += ia22_8 * rx8;
         ry23 -= ia23_8 * ix8;
         iy23 += ia23_8 * rx8;

         ry0 += ra0_9 * rx9;
         iy0 += ra0_9 * ix9;
         ry1 += ra1_9 * rx9;
         iy1 += ra1_9 * ix9;
         ry2 += ra2_9 * rx9;
         iy2 += ra2_9 * ix9;
         ry3 += ra3_9 * rx9;
         iy3 += ra3_9 * ix9;
         ry4 += ra4_9 * rx9;
         iy4 += ra4_9 * ix9;
         ry5 += ra5_9 * rx9;
         iy5 += ra5_9 * ix9;
         ry6 += ra6_9 * rx9;
         iy6 += ra6_9 * ix9;
         ry7 += ra7_9 * rx9;
         iy7 += ra7_9 * ix9;
         ry8 += ra8_9 * rx9;
         iy8 += ra8_9 * ix9;
         ry9 += ra9_9 * rx9;
         iy9 += ra9_9 * ix9;
         ry10 += ra10_9 * rx9;
         iy10 += ra10_9 * ix9;
         ry11 += ra11_9 * rx9;
         iy11 += ra11_9 * ix9;
         ry12 += ra12_9 * rx9;
         iy12 += ra12_9 * ix9;
         ry13 += ra13_9 * rx9;
         iy13 += ra13_9 * ix9;
         ry14 += ra14_9 * rx9;
         iy14 += ra14_9 * ix9;
         ry15 += ra15_9 * rx9;
         iy15 += ra15_9 * ix9;
         ry16 += ra16_9 * rx9;
         iy16 += ra16_9 * ix9;
         ry17 += ra17_9 * rx9;
         iy17 += ra17_9 * ix9;
         ry18 += ra18_9 * rx9;
         iy18 += ra18_9 * ix9;
         ry19 += ra19_9 * rx9;
         iy19 += ra19_9 * ix9;
         ry20 += ra20_9 * rx9;
         iy20 += ra20_9 * ix9;
         ry21 += ra21_9 * rx9;
         iy21 += ra21_9 * ix9;
         ry22 += ra22_9 * rx9;
         iy22 += ra22_9 * ix9;
         ry23 += ra23_9 * rx9;
         iy23 += ra23_9 * ix9;
         ry0 -= ia0_9 * ix9;
         iy0 += ia0_9 * rx9;
         prefA(A9);
         ry1 -= ia1_9 * ix9;
         iy1 += ia1_9 * rx9;
         ry2 -= ia2_9 * ix9;
         iy2 += ia2_9 * rx9;
         ry3 -= ia3_9 * ix9;
         iy3 += ia3_9 * rx9;
         ry4 -= ia4_9 * ix9;
         iy4 += ia4_9 * rx9;
         ry5 -= ia5_9 * ix9;
         iy5 += ia5_9 * rx9;
         ry6 -= ia6_9 * ix9;
         iy6 += ia6_9 * rx9;
         ry7 -= ia7_9 * ix9;
         iy7 += ia7_9 * rx9;
         ry8 -= ia8_9 * ix9;
         iy8 += ia8_9 * rx9;
         prefA(A9);
         ry9 -= ia9_9 * ix9;
         iy9 += ia9_9 * rx9;
         ry10 -= ia10_9 * ix9;
         iy10 += ia10_9 * rx9;
         ry11 -= ia11_9 * ix9;
         iy11 += ia11_9 * rx9;
         ry12 -= ia12_9 * ix9;
         iy12 += ia12_9 * rx9;
         ry13 -= ia13_9 * ix9;
         iy13 += ia13_9 * rx9;
         ry14 -= ia14_9 * ix9;
         iy14 += ia14_9 * rx9;
         ry15 -= ia15_9 * ix9;
         iy15 += ia15_9 * rx9;
         ry16 -= ia16_9 * ix9;
         iy16 += ia16_9 * rx9;
         prefA(A9);
         ry17 -= ia17_9 * ix9;
         iy17 += ia17_9 * rx9;
         ry18 -= ia18_9 * ix9;
         iy18 += ia18_9 * rx9;
         ry19 -= ia19_9 * ix9;
         iy19 += ia19_9 * rx9;
         ry20 -= ia20_9 * ix9;
         iy20 += ia20_9 * rx9;
         ry21 -= ia21_9 * ix9;
         iy21 += ia21_9 * rx9;
         ry22 -= ia22_9 * ix9;
         iy22 += ia22_9 * rx9;
         ry23 -= ia23_9 * ix9;
         iy23 += ia23_9 * rx9;

         ry0 += ra0_10 * rx10;
         iy0 += ra0_10 * ix10;
         ry1 += ra1_10 * rx10;
         iy1 += ra1_10 * ix10;
         ry2 += ra2_10 * rx10;
         iy2 += ra2_10 * ix10;
         ry3 += ra3_10 * rx10;
         iy3 += ra3_10 * ix10;
         ry4 += ra4_10 * rx10;
         iy4 += ra4_10 * ix10;
         ry5 += ra5_10 * rx10;
         iy5 += ra5_10 * ix10;
         ry6 += ra6_10 * rx10;
         iy6 += ra6_10 * ix10;
         ry7 += ra7_10 * rx10;
         iy7 += ra7_10 * ix10;
         ry8 += ra8_10 * rx10;
         iy8 += ra8_10 * ix10;
         ry9 += ra9_10 * rx10;
         iy9 += ra9_10 * ix10;
         ry10 += ra10_10 * rx10;
         iy10 += ra10_10 * ix10;
         ry11 += ra11_10 * rx10;
         iy11 += ra11_10 * ix10;
         ry12 += ra12_10 * rx10;
         iy12 += ra12_10 * ix10;
         ry13 += ra13_10 * rx10;
         iy13 += ra13_10 * ix10;
         ry14 += ra14_10 * rx10;
         iy14 += ra14_10 * ix10;
         ry15 += ra15_10 * rx10;
         iy15 += ra15_10 * ix10;
         ry16 += ra16_10 * rx10;
         iy16 += ra16_10 * ix10;
         ry17 += ra17_10 * rx10;
         iy17 += ra17_10 * ix10;
         ry18 += ra18_10 * rx10;
         iy18 += ra18_10 * ix10;
         ry19 += ra19_10 * rx10;
         iy19 += ra19_10 * ix10;
         ry20 += ra20_10 * rx10;
         iy20 += ra20_10 * ix10;
         ry21 += ra21_10 * rx10;
         iy21 += ra21_10 * ix10;
         ry22 += ra22_10 * rx10;
         iy22 += ra22_10 * ix10;
         ry23 += ra23_10 * rx10;
         iy23 += ra23_10 * ix10;
         ry0 -= ia0_10 * ix10;
         iy0 += ia0_10 * rx10;
         prefA(A10);
         ry1 -= ia1_10 * ix10;
         iy1 += ia1_10 * rx10;
         ry2 -= ia2_10 * ix10;
         iy2 += ia2_10 * rx10;
         ry3 -= ia3_10 * ix10;
         iy3 += ia3_10 * rx10;
         ry4 -= ia4_10 * ix10;
         iy4 += ia4_10 * rx10;
         ry5 -= ia5_10 * ix10;
         iy5 += ia5_10 * rx10;
         ry6 -= ia6_10 * ix10;
         iy6 += ia6_10 * rx10;
         ry7 -= ia7_10 * ix10;
         iy7 += ia7_10 * rx10;
         ry8 -= ia8_10 * ix10;
         iy8 += ia8_10 * rx10;
         prefA(A10);
         ry9 -= ia9_10 * ix10;
         iy9 += ia9_10 * rx10;
         ry10 -= ia10_10 * ix10;
         iy10 += ia10_10 * rx10;
         ry11 -= ia11_10 * ix10;
         iy11 += ia11_10 * rx10;
         ry12 -= ia12_10 * ix10;
         iy12 += ia12_10 * rx10;
         ry13 -= ia13_10 * ix10;
         iy13 += ia13_10 * rx10;
         ry14 -= ia14_10 * ix10;
         iy14 += ia14_10 * rx10;
         ry15 -= ia15_10 * ix10;
         iy15 += ia15_10 * rx10;
         ry16 -= ia16_10 * ix10;
         iy16 += ia16_10 * rx10;
         prefA(A10);
         ry17 -= ia17_10 * ix10;
         iy17 += ia17_10 * rx10;
         ry18 -= ia18_10 * ix10;
         iy18 += ia18_10 * rx10;
         ry19 -= ia19_10 * ix10;
         iy19 += ia19_10 * rx10;
         ry20 -= ia20_10 * ix10;
         iy20 += ia20_10 * rx10;
         ry21 -= ia21_10 * ix10;
         iy21 += ia21_10 * rx10;
         ry22 -= ia22_10 * ix10;
         iy22 += ia22_10 * rx10;
         ry23 -= ia23_10 * ix10;
         iy23 += ia23_10 * rx10;

         ry0 += ra0_11 * rx11;
         iy0 += ra0_11 * ix11;
         ry1 += ra1_11 * rx11;
         iy1 += ra1_11 * ix11;
         ry2 += ra2_11 * rx11;
         iy2 += ra2_11 * ix11;
         ry3 += ra3_11 * rx11;
         iy3 += ra3_11 * ix11;
         ry4 += ra4_11 * rx11;
         iy4 += ra4_11 * ix11;
         ry5 += ra5_11 * rx11;
         iy5 += ra5_11 * ix11;
         ry6 += ra6_11 * rx11;
         iy6 += ra6_11 * ix11;
         ry7 += ra7_11 * rx11;
         iy7 += ra7_11 * ix11;
         ry8 += ra8_11 * rx11;
         iy8 += ra8_11 * ix11;
         ry9 += ra9_11 * rx11;
         iy9 += ra9_11 * ix11;
         ry10 += ra10_11 * rx11;
         iy10 += ra10_11 * ix11;
         ry11 += ra11_11 * rx11;
         iy11 += ra11_11 * ix11;
         ry12 += ra12_11 * rx11;
         iy12 += ra12_11 * ix11;
         ry13 += ra13_11 * rx11;
         iy13 += ra13_11 * ix11;
         ry14 += ra14_11 * rx11;
         iy14 += ra14_11 * ix11;
         ry15 += ra15_11 * rx11;
         iy15 += ra15_11 * ix11;
         ry16 += ra16_11 * rx11;
         iy16 += ra16_11 * ix11;
         ry17 += ra17_11 * rx11;
         iy17 += ra17_11 * ix11;
         ry18 += ra18_11 * rx11;
         iy18 += ra18_11 * ix11;
         ry19 += ra19_11 * rx11;
         iy19 += ra19_11 * ix11;
         ry20 += ra20_11 * rx11;
         iy20 += ra20_11 * ix11;
         ry21 += ra21_11 * rx11;
         iy21 += ra21_11 * ix11;
         ry22 += ra22_11 * rx11;
         iy22 += ra22_11 * ix11;
         ry23 += ra23_11 * rx11;
         iy23 += ra23_11 * ix11;
         ry0 -= ia0_11 * ix11;
         iy0 += ia0_11 * rx11;
         prefA(A11);
         ry1 -= ia1_11 * ix11;
         iy1 += ia1_11 * rx11;
         ry2 -= ia2_11 * ix11;
         iy2 += ia2_11 * rx11;
         ry3 -= ia3_11 * ix11;
         iy3 += ia3_11 * rx11;
         ry4 -= ia4_11 * ix11;
         iy4 += ia4_11 * rx11;
         ry5 -= ia5_11 * ix11;
         iy5 += ia5_11 * rx11;
         ry6 -= ia6_11 * ix11;
         iy6 += ia6_11 * rx11;
         ry7 -= ia7_11 * ix11;
         iy7 += ia7_11 * rx11;
         ry8 -= ia8_11 * ix11;
         iy8 += ia8_11 * rx11;
         prefA(A11);
         ry9 -= ia9_11 * ix11;
         iy9 += ia9_11 * rx11;
         ry10 -= ia10_11 * ix11;
         iy10 += ia10_11 * rx11;
         ry11 -= ia11_11 * ix11;
         iy11 += ia11_11 * rx11;
         ry12 -= ia12_11 * ix11;
         iy12 += ia12_11 * rx11;
         ry13 -= ia13_11 * ix11;
         iy13 += ia13_11 * rx11;
         ry14 -= ia14_11 * ix11;
         iy14 += ia14_11 * rx11;
         ry15 -= ia15_11 * ix11;
         iy15 += ia15_11 * rx11;
         ry16 -= ia16_11 * ix11;
         iy16 += ia16_11 * rx11;
         prefA(A11);
         ry17 -= ia17_11 * ix11;
         iy17 += ia17_11 * rx11;
         ry18 -= ia18_11 * ix11;
         iy18 += ia18_11 * rx11;
         ry19 -= ia19_11 * ix11;
         iy19 += ia19_11 * rx11;
         ry20 -= ia20_11 * ix11;
         iy20 += ia20_11 * rx11;
         ry21 -= ia21_11 * ix11;
         iy21 += ia21_11 * rx11;
         ry22 -= ia22_11 * ix11;
         iy22 += ia22_11 * rx11;
         ry23 -= ia23_11 * ix11;
         iy23 += ia23_11 * rx11;


         Y[i+0] = ry0;
         Y[i+1] = iy0;
         Y[i+2] = ry1;
         Y[i+3] = iy1;
         Y[i+4] = ry2;
         Y[i+5] = iy2;
         Y[i+6] = ry3;
         Y[i+7] = iy3;
         Y[i+8] = ry4;
         Y[i+9] = iy4;
         Y[i+10] = ry5;
         Y[i+11] = iy5;
         Y[i+12] = ry6;
         Y[i+13] = iy6;
         Y[i+14] = ry7;
         Y[i+15] = iy7;
         Y[i+16] = ry8;
         Y[i+17] = iy8;
         Y[i+18] = ry9;
         Y[i+19] = iy9;
         Y[i+20] = ry10;
         Y[i+21] = iy10;
         Y[i+22] = ry11;
         Y[i+23] = iy11;
         Y[i+24] = ry12;
         Y[i+25] = iy12;
         Y[i+26] = ry13;
         Y[i+27] = iy13;
         Y[i+28] = ry14;
         Y[i+29] = iy14;
         Y[i+30] = ry15;
         Y[i+31] = iy15;
         Y[i+32] = ry16;
         Y[i+33] = iy16;
         Y[i+34] = ry17;
         Y[i+35] = iy17;
         Y[i+36] = ry18;
         Y[i+37] = iy18;
         Y[i+38] = ry19;
         Y[i+39] = iy19;
         Y[i+40] = ry20;
         Y[i+41] = iy20;
         Y[i+42] = ry21;
         Y[i+43] = iy21;
         Y[i+44] = ry22;
         Y[i+45] = iy22;
         Y[i+46] = ry23;
         Y[i+47] = iy23;
      }
      for (i=M24; i < M; i += 2)
      {
         register float ry0 = Y[i], iy0 = Y[i+1];
         const float ra0_0=A0[i], ia0_0=A0[i+1], ra0_1=A1[i], ia0_1=A1[i+1],
                     ra0_2=A2[i], ia0_2=A2[i+1], ra0_3=A3[i], ia0_3=A3[i+1],
                     ra0_4=A4[i], ia0_4=A4[i+1], ra0_5=A5[i], ia0_5=A5[i+1],
                     ra0_6=A6[i], ia0_6=A6[i+1], ra0_7=A7[i], ia0_7=A7[i+1],
                     ra0_8=A8[i], ia0_8=A8[i+1], ra0_9=A9[i], ia0_9=A9[i+1],
                     ra0_10=A10[i], ia0_10=A10[i+1], ra0_11=A11[i],
                     ia0_11=A11[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         ry0 += ra0_1 * rx1;
         iy0 += ra0_1 * ix1;
         ry0 -= ia0_1 * ix1;
         iy0 += ia0_1 * rx1;
         ry0 += ra0_2 * rx2;
         iy0 += ra0_2 * ix2;
         ry0 -= ia0_2 * ix2;
         iy0 += ia0_2 * rx2;
         ry0 += ra0_3 * rx3;
         iy0 += ra0_3 * ix3;
         ry0 -= ia0_3 * ix3;
         iy0 += ia0_3 * rx3;
         ry0 += ra0_4 * rx4;
         iy0 += ra0_4 * ix4;
         ry0 -= ia0_4 * ix4;
         iy0 += ia0_4 * rx4;
         ry0 += ra0_5 * rx5;
         iy0 += ra0_5 * ix5;
         ry0 -= ia0_5 * ix5;
         iy0 += ia0_5 * rx5;
         ry0 += ra0_6 * rx6;
         iy0 += ra0_6 * ix6;
         ry0 -= ia0_6 * ix6;
         iy0 += ia0_6 * rx6;
         ry0 += ra0_7 * rx7;
         iy0 += ra0_7 * ix7;
         ry0 -= ia0_7 * ix7;
         iy0 += ia0_7 * rx7;
         ry0 += ra0_8 * rx8;
         iy0 += ra0_8 * ix8;
         ry0 -= ia0_8 * ix8;
         iy0 += ia0_8 * rx8;
         ry0 += ra0_9 * rx9;
         iy0 += ra0_9 * ix9;
         ry0 -= ia0_9 * ix9;
         iy0 += ia0_9 * rx9;
         ry0 += ra0_10 * rx10;
         iy0 += ra0_10 * ix10;
         ry0 -= ia0_10 * ix10;
         iy0 += ia0_10 * rx10;
         ry0 += ra0_11 * rx11;
         iy0 += ra0_11 * ix11;
         ry0 -= ia0_11 * ix11;
         iy0 += ia0_11 * rx11;
         Y[i] = ry0;
         Y[i+1] = iy0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N12; j; j--, A += lda, X += 2)
   {
      const float *A0=A;
      register float rx0=X[0], ix0=X[1];
      ATL_INT i;
      prefY(X+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2], ia1_0=A0[i+3],
                     ra2_0=A0[i+4], ia2_0=A0[i+5], ra3_0=A0[i+6], ia3_0=A0[i+7],
                     ra4_0=A0[i+8], ia4_0=A0[i+9], ra5_0=A0[i+10],
                     ia5_0=A0[i+11], ra6_0=A0[i+12], ia6_0=A0[i+13],
                     ra7_0=A0[i+14], ia7_0=A0[i+15], ra8_0=A0[i+16],
                     ia8_0=A0[i+17], ra9_0=A0[i+18], ia9_0=A0[i+19],
                     ra10_0=A0[i+20], ia10_0=A0[i+21], ra11_0=A0[i+22],
                     ia11_0=A0[i+23], ra12_0=A0[i+24], ia12_0=A0[i+25],
                     ra13_0=A0[i+26], ia13_0=A0[i+27], ra14_0=A0[i+28],
                     ia14_0=A0[i+29], ra15_0=A0[i+30], ia15_0=A0[i+31],
                     ra16_0=A0[i+32], ia16_0=A0[i+33], ra17_0=A0[i+34],
                     ia17_0=A0[i+35], ra18_0=A0[i+36], ia18_0=A0[i+37],
                     ra19_0=A0[i+38], ia19_0=A0[i+39], ra20_0=A0[i+40],
                     ia20_0=A0[i+41], ra21_0=A0[i+42], ia21_0=A0[i+43],
                     ra22_0=A0[i+44], ia22_0=A0[i+45], ra23_0=A0[i+46],
                     ia23_0=A0[i+47];
         register float ry0=Y[i+0], iy0=Y[i+1], ry1=Y[i+2], iy1=Y[i+3],
                        ry2=Y[i+4], iy2=Y[i+5], ry3=Y[i+6], iy3=Y[i+7],
                        ry4=Y[i+8], iy4=Y[i+9], ry5=Y[i+10], iy5=Y[i+11],
                        ry6=Y[i+12], iy6=Y[i+13], ry7=Y[i+14], iy7=Y[i+15],
                        ry8=Y[i+16], iy8=Y[i+17], ry9=Y[i+18], iy9=Y[i+19],
                        ry10=Y[i+20], iy10=Y[i+21], ry11=Y[i+22], iy11=Y[i+23],
                        ry12=Y[i+24], iy12=Y[i+25], ry13=Y[i+26], iy13=Y[i+27],
                        ry14=Y[i+28], iy14=Y[i+29], ry15=Y[i+30], iy15=Y[i+31],
                        ry16=Y[i+32], iy16=Y[i+33], ry17=Y[i+34], iy17=Y[i+35],
                        ry18=Y[i+36], iy18=Y[i+37], ry19=Y[i+38], iy19=Y[i+39],
                        ry20=Y[i+40], iy20=Y[i+41], ry21=Y[i+42], iy21=Y[i+43],
                        ry22=Y[i+44], iy22=Y[i+45], ry23=Y[i+46], iy23=Y[i+47];

         prefY(Y);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra1_0 * rx0;
         iy1 += ra1_0 * ix0;
         ry2 += ra2_0 * rx0;
         iy2 += ra2_0 * ix0;
         ry3 += ra3_0 * rx0;
         iy3 += ra3_0 * ix0;
         ry4 += ra4_0 * rx0;
         iy4 += ra4_0 * ix0;
         ry5 += ra5_0 * rx0;
         iy5 += ra5_0 * ix0;
         ry6 += ra6_0 * rx0;
         iy6 += ra6_0 * ix0;
         ry7 += ra7_0 * rx0;
         iy7 += ra7_0 * ix0;
         ry8 += ra8_0 * rx0;
         iy8 += ra8_0 * ix0;
         ry9 += ra9_0 * rx0;
         iy9 += ra9_0 * ix0;
         ry10 += ra10_0 * rx0;
         iy10 += ra10_0 * ix0;
         ry11 += ra11_0 * rx0;
         iy11 += ra11_0 * ix0;
         ry12 += ra12_0 * rx0;
         iy12 += ra12_0 * ix0;
         ry13 += ra13_0 * rx0;
         iy13 += ra13_0 * ix0;
         ry14 += ra14_0 * rx0;
         iy14 += ra14_0 * ix0;
         ry15 += ra15_0 * rx0;
         iy15 += ra15_0 * ix0;
         ry16 += ra16_0 * rx0;
         iy16 += ra16_0 * ix0;
         ry17 += ra17_0 * rx0;
         iy17 += ra17_0 * ix0;
         ry18 += ra18_0 * rx0;
         iy18 += ra18_0 * ix0;
         ry19 += ra19_0 * rx0;
         iy19 += ra19_0 * ix0;
         ry20 += ra20_0 * rx0;
         iy20 += ra20_0 * ix0;
         ry21 += ra21_0 * rx0;
         iy21 += ra21_0 * ix0;
         ry22 += ra22_0 * rx0;
         iy22 += ra22_0 * ix0;
         ry23 += ra23_0 * rx0;
         iy23 += ra23_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry1 -= ia1_0 * ix0;
         iy1 += ia1_0 * rx0;
         ry2 -= ia2_0 * ix0;
         iy2 += ia2_0 * rx0;
         ry3 -= ia3_0 * ix0;
         iy3 += ia3_0 * rx0;
         ry4 -= ia4_0 * ix0;
         iy4 += ia4_0 * rx0;
         ry5 -= ia5_0 * ix0;
         iy5 += ia5_0 * rx0;
         ry6 -= ia6_0 * ix0;
         iy6 += ia6_0 * rx0;
         ry7 -= ia7_0 * ix0;
         iy7 += ia7_0 * rx0;
         ry8 -= ia8_0 * ix0;
         iy8 += ia8_0 * rx0;
         prefA(A0);
         ry9 -= ia9_0 * ix0;
         iy9 += ia9_0 * rx0;
         ry10 -= ia10_0 * ix0;
         iy10 += ia10_0 * rx0;
         ry11 -= ia11_0 * ix0;
         iy11 += ia11_0 * rx0;
         ry12 -= ia12_0 * ix0;
         iy12 += ia12_0 * rx0;
         ry13 -= ia13_0 * ix0;
         iy13 += ia13_0 * rx0;
         ry14 -= ia14_0 * ix0;
         iy14 += ia14_0 * rx0;
         ry15 -= ia15_0 * ix0;
         iy15 += ia15_0 * rx0;
         ry16 -= ia16_0 * ix0;
         iy16 += ia16_0 * rx0;
         prefA(A0);
         ry17 -= ia17_0 * ix0;
         iy17 += ia17_0 * rx0;
         ry18 -= ia18_0 * ix0;
         iy18 += ia18_0 * rx0;
         ry19 -= ia19_0 * ix0;
         iy19 += ia19_0 * rx0;
         ry20 -= ia20_0 * ix0;
         iy20 += ia20_0 * rx0;
         ry21 -= ia21_0 * ix0;
         iy21 += ia21_0 * rx0;
         ry22 -= ia22_0 * ix0;
         iy22 += ia22_0 * rx0;
         ry23 -= ia23_0 * ix0;
         iy23 += ia23_0 * rx0;


         Y[i+0] = ry0;
         Y[i+1] = iy0;
         Y[i+2] = ry1;
         Y[i+3] = iy1;
         Y[i+4] = ry2;
         Y[i+5] = iy2;
         Y[i+6] = ry3;
         Y[i+7] = iy3;
         Y[i+8] = ry4;
         Y[i+9] = iy4;
         Y[i+10] = ry5;
         Y[i+11] = iy5;
         Y[i+12] = ry6;
         Y[i+13] = iy6;
         Y[i+14] = ry7;
         Y[i+15] = iy7;
         Y[i+16] = ry8;
         Y[i+17] = iy8;
         Y[i+18] = ry9;
         Y[i+19] = iy9;
         Y[i+20] = ry10;
         Y[i+21] = iy10;
         Y[i+22] = ry11;
         Y[i+23] = iy11;
         Y[i+24] = ry12;
         Y[i+25] = iy12;
         Y[i+26] = ry13;
         Y[i+27] = iy13;
         Y[i+28] = ry14;
         Y[i+29] = iy14;
         Y[i+30] = ry15;
         Y[i+31] = iy15;
         Y[i+32] = ry16;
         Y[i+33] = iy16;
         Y[i+34] = ry17;
         Y[i+35] = iy17;
         Y[i+36] = ry18;
         Y[i+37] = iy18;
         Y[i+38] = ry19;
         Y[i+39] = iy19;
         Y[i+40] = ry20;
         Y[i+41] = iy20;
         Y[i+42] = ry21;
         Y[i+43] = iy21;
         Y[i+44] = ry22;
         Y[i+45] = iy22;
         Y[i+46] = ry23;
         Y[i+47] = iy23;
      }
      for (i=M24; i < M; i += 2)
      {
         register float ry0 = Y[i], iy0 = Y[i+1];
         const float ra0_0=A0[i], ia0_0=A0[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         Y[i] = ry0;
         Y[i+1] = iy0;
      }
   }
}
